// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.

#pragma once

# include "mscrypt.h"
# include <openssl/bio.h>

#include <gio/gio.h>

#ifdef  __cplusplus
extern "C" {
#endif



int MSCRYPT_GDBUS_open_shared_mem(
    MSCRYPT_SHARED_MEM *sharedMem);

void MSCRYPT_GDBUS_close_shared_mem(
    MSCRYPT_SHARED_MEM *sharedMem);

int MSCRYPT_GDBUS_CLIENT_cert_ctrl(
    const uuid_t correlationId,
    MSCRYPT_SHARED_MEM *sharedMem,
    int ctrl,
    int location,
    int format,
    int length,
    const unsigned char *bytes);

int MSCRYPT_GDBUS_CLIENT_import_pfx(
    const uuid_t correlationId,
    int mscryptFlags,
    int inPfxLength,
    const unsigned char *inPfxBytes,
    const char *inPassword,             // Optional
    int *outVerifyChainError,
    int *outPfxLength,
    unsigned char **outPfxBytes,        // MSCRYPT_free()
    char **outPfxSalt);                 // MSCRYPT_free()

int MSCRYPT_GDBUS_CLIENT_pfx_open(
    MSCRYPT_PFX_CTX *keyCtx,
    int pfxLength,
    const unsigned char *pfxBytes,
    const char *salt);


void MSCRYPT_GDBUS_CLIENT_pfx_close(
    MSCRYPT_PFX_CTX *keyCtx);


int MSCRYPT_GDBUS_CLIENT_rsa_private_encrypt_decrypt(
    MSCRYPT_PFX_CTX *keyCtx,
    int decrypt,
    int flen,
    const unsigned char *from,
    int tlen,
    unsigned char *to,
    int padding);

int MSCRYPT_GDBUS_CLIENT_ecdsa_sign(
    MSCRYPT_PFX_CTX *keyCtx,
    int type,
    const unsigned char *dgst,
    int dlen,
    unsigned char *sig,
    unsigned int siglen,
    unsigned int *outlen);

int MSCRYPT_GDBUS_CLIENT_create_self_sign_pfx(
    const uuid_t correlationId,
    int mscryptFlags,
    const char *confStr,
    int *outPfxLength,
    unsigned char **outPfxBytes,        // MSCRYPT_free()
    char **outPfxSalt);                 // MSCRYPT_free()

// The pemCert consists of the end certificate followed by
// 1 or more CA certificates
int MSCRYPT_GDBUS_CLIENT_replace_pfx_certs(
    const uuid_t correlationId,
    int mscryptFlags,
    int inPfxLength,
    const unsigned char *inPfxBytes,
    const char *inSalt,
    int pemCertLength,
    const unsigned char *pemCertBytes,
    int *outPfxLength,
    unsigned char **outPfxBytes,    // MSCRYPT_free()
    char **outSalt);                // MSCRYPT_free()

// Before returning
//  g_error_free(*error)
//  *error = NULL;
void _MSCRYPT_GDBUS_trace_log_glib_error_para(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    GError **error,           // Before returning, g_error_free(*error)
    const char *format, ...);
#define MSCRYPT_GDBUS_trace_log_glib_error_para(correlationId, flags, title, loc, error, ...) \
    _MSCRYPT_GDBUS_trace_log_glib_error_para(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc, error, __VA_ARGS__)

// Before returning
//  g_error_free(*error)
//  *error = NULL;
void _MSCRYPT_GDBUS_trace_log_glib_error(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    GError **error);          // Before returning, g_error_free(*error)
#define MSCRYPT_GDBUS_trace_log_glib_error(correlationId, flags, title, loc, error) \
    _MSCRYPT_GDBUS_trace_log_glib_error(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc, error)


#ifdef  __cplusplus
}
#endif
